# DeeplabV3 Evaluation

## Environment
- Tensorflow 1.15 for training/ckpt, pb evaluation
- Tensorflow 2.2 for post-training quantization/tflite evaluation
- python3.6


## Evaluation

### Evaluate ckpt

To use official evaluation file, please use `eval.py` which will evaluate the result in the folder non-stop.

```bash
python eval.py \
    --checkpoint_dir=${CKPT_DIR} \
    --dataset_dir=${DATA_DIR} \
    --eval_logdir=log \
    --eval_crop_size="512,512" \
    --min_resize_value=512 \
    --max_resize_value=512 \
    --decoder_output_stride=4 \
    --resize_factor=16 \
    --output_stride=16
```

- CKPT_DIR: The folder where your ckpt is.
- DATA_DIR: Dataset folder, where the tfrecord is.

### Evaluate  GraphDef (pb)

- Export pb from ckpt

```bash
python export_model.py \
    --checkpoint_path=${CKPT_DIR} \
    --export_path=${PB_OUTPUT_DIR} \
    --num_classes=32 \
    --output_stride=16 \
    --decoder_output_stride=4 \
    --train_crop_size="512,512" \
    --min_resize_value=512 \
    --max_resize_value=512 \
    --resize_factor=16
```
- CKPT_DIR: The folder where your ckpt is.
- PB_OUTPUT_DIR: The folder where output pb will be.

- Evaluate pb

```bash
python eval_pb.py \
    --pb=${PB_FILE} \
    --dataset_dir=${DATA_DIR} \
    --eval_logdir=log \
    --eval_crop_size="512,512" \
    --min_resize_value=512 \
    --max_resize_value=512 \
    --resize_factor=16

```

- PB_FILE: The path of your pb file.
- DATA_DIR: Dataset folder, where the tfrecord is.

### Evaluate TFLite

- Generate data for TFLite evaluation.

```bash
python gen_preprocessed_data.py \
  --eval_crop_size='512,512' \
  --min_resize_value=512 \
  --max_resize_value=512 \
  --dataset_dir=${DATA_DIR} \
  --preprocessed_dataset_dir=${OUTPUT_DIR}\
  --eval_split=val
```
- DATA_DIR: Dataset folder, where the tfrecord is.
- OUTPUT_DIR: Dataset folder, where the preprocessed data will be stored. A `images` and `annotations` folder will be created under this folder.


- Evaluate TFLite
```bash
python segmentation_inference_eval_seg_tflite.py
```

Note that please modify the index in the code.

- test_dir = ${INPUT_DATA_DIR}
- val_label_dir = ${ANNOTATION_DATA_DIR}
- net_file = ${TFLITE_FILE}
- is_float_input = ${IS_FLOAT}
- is_input_0_255 = ${IS_INPUT_0_255}

- INPUT_DATA_DIR: Input data folder containing input image.
- ANNOTATION_DATA_DIR: Annotation data folder containnig annotation image.
- NET_FILE: Path to TFLite file to be evaluate.
- IS_FLOAT: A bool indicates input type is float or uint8.
- IS_INPUT_0_255: A bool indicates input of the model is [0, 255] or [-1, 1]

- **Note**: To evaluate our model:
  - `checkpoints/float/freeze.tflite`
    - IS_FLOAT = True
    - IS_INPUT_0_255 = False
  - `checkpoints/post_train_quant/first500.tflite`
    - IS_FLOAT = False
    - IS_INPUT_0_255 = True

## Post-training quantization

### Generate data

- Generate data for TFLite evaluation.

1. Generate tfrecord with first 500 training images in ADE20K for post-training quantization.
You can find list in `checkpoints/post_train_quant/quant.txt`

```bash
CUDA_VISIBLE_DEVICES=2 python build_ade20k_data.py \
  --train_image_folder=${TRAIN_IMAGE_FOLDER} \
  --train_image_label_folder=${TRAIN_LABEL_FOLDER} \
  --val_image_folder=${VAL_IMAGE_FOLDER} \
  --val_image_label_folder=${VAL_LABEL_FOLDER} \
  --output_dir=${OUTPUT_DIR}
```
- TRAIN_IMAGE_FOLDER: The folder with first 500 training images downloaded from ADEChallengeData2016 inside.
- TRAIN_LABEL_FOLDER:The folder with first 500 training labels downloaded from ADEChallengeData2016 inside.
- VAL_IMAGE_FOLDER: The folder with 2000 validation images downloaded from ADEChallengeData2016 inside.
- VAL_LABEL_FOLDER: The folder with 2000 validation labels downloaded from ADEChallengeData2016 inside.
- OUTPUT_DIR: The folder where the output tfrecord will be.

2. Gen Preprocessed data for quantization. 
```bash
python gen_preprocessed_data.py \
  --eval_crop_size='512,512' \
  --min_resize_value=512 \
  --max_resize_value=512 \
  --dataset_dir=${DATA_DIR} \
  --preprocessed_dataset_dir=${OUTPUT_DIR}\
  --eval_split=train
```
- DATA_DIR: Dataset folder. Exactly the `TRAIN_IMAGE_FOLDER` in previous step.
- OUTPUT_DIR: Dataset folder, where the preprocessed data will be stored. A `images` and `annotations` folder will be created under this folder.

3. Post-training quantization

```bash
python3.6 post_training_quant.py \
    --dataset_dir=${PREPROCESSED_DATA_DIR} \
    --graph_def=${INPUT_PB_FILE} \
    --out_tflite=${OUTPUT_TFLITE_FILE} \
    --input_type=uint8 \
    --sample_num=500
```
- PREPROCESSED_DATA_DIR: Dataset folder. Exactly the `OUTPUT_DIR` in previous step.
- INPUT_PB_FILE: The path of your Graph def (.pb) to be quantized.
- OUTPUT_TFLITE_FILE: The path of your output tflite file.

